@@ -67,6 +67,7 @@ protected:
 	bool need_redirect(ParsedURI& uri);
 	bool redirect_url(HttpResponse *client_resp, ParsedURI& uri);
 	void set_empty_request();
+	void check_response();
 
 private:
 	int redirect_max_;
@@ -159,7 +160,7 @@ CommMessageOut *ComplexHttpTask::message_out()
 
 CommMessageIn *ComplexHttpTask::message_in()
 {
-	auto *resp = this->get_resp();
+	HttpResponse *resp = this->get_resp();
 
 	if (strcmp(this->get_req()->get_method(), HttpMethodHead) == 0)
 		resp->parse_zero_body();
@@ -334,17 +335,37 @@ bool ComplexHttpTask::need_redirect(ParsedURI& uri)
 	return false;
 }
 
+void ComplexHttpTask::check_response()
+{
+	HttpResponse *resp = this->get_resp();
+
+	resp->end_parsing();
+	if (this->state == WFT_STATE_SYS_ERROR && this->error == ECONNRESET)
+	{
+		/* Servers can end the message by closing the connection. */
+		if (resp->is_header_complete() &&
+			!resp->is_keep_alive() &&
+			!resp->is_chunked() &&
+			!resp->has_content_length_header())
+		{
+			this->state = WFT_STATE_SUCCESS;
+			this->error = 0;
+		}
+	}
+}
+
 bool ComplexHttpTask::finish_once()
 {
+	if (this->state != WFT_STATE_SUCCESS)
+		this->check_response();
+
 	if (this->state == WFT_STATE_SUCCESS)
 	{
-		if (need_redirect(uri_))
+		if (this->need_redirect(uri_))
 			this->set_redirect(uri_);
 		else if (this->state != WFT_STATE_SUCCESS)
 			this->disable_retry();
 	}
-	else
-		this->get_resp()->end_parsing();
 
 	return true;
 }
@@ -353,38 +374,19 @@ bool ComplexHttpTask::finish_once()
 
 static int __encode_auth(const char *p, std::string& auth)
 {
-	static SSL_CTX *init_ssl = WFGlobal::get_ssl_client_ctx();
-	(void)init_ssl;
-	BUF_MEM *bptr;
-	BIO *bmem;
-	BIO *b64;
+	size_t len = strlen(p);
+	size_t base64_len = (len + 2) / 3 * 4;
+	char *base64 = (char *)malloc(base64_len + 1);
 
-	b64 = BIO_new(BIO_f_base64());
-	if (b64)
-	{
-		bmem = BIO_new(BIO_s_mem());
-		if (bmem)
-		{
-			BIO_set_flags(b64, BIO_FLAGS_BASE64_NO_NL);
-			b64 = BIO_push(b64, bmem);
-			BIO_write(b64, p, strlen(p));
-			(void)BIO_flush(b64);
-			BIO_get_mem_ptr(b64, &bptr);
+	if (!base64)
+		return -1;
 
-			if (bptr->length > 0)
-			{
-				auth.append("Basic ");
-				auth.append(bptr->data, bptr->length);
-			}
+	EVP_EncodeBlock((unsigned char *)base64, (const unsigned char *)p, len);
+	auth.append("Basic ");
+	auth.append(base64, base64_len);
 
-			BIO_free_all(b64);
-			return 0;
-		}
-
-		BIO_free_all(b64);
-	}
-
-	return -1;
+	free(base64);
+	return 0;
 }
 
 static SSL *__create_ssl(SSL_CTX *ssl_ctx)
@@ -729,6 +731,9 @@ bool ComplexHttpProxyTask::finish_once()
 		return false;
 	}
 
+	if (this->state != WFT_STATE_SUCCESS)
+		this->check_response();
+
 	if (this->state == WFT_STATE_SUCCESS)
 	{
 		if (this->need_redirect(user_uri_))
@@ -736,8 +741,6 @@ bool ComplexHttpProxyTask::finish_once()
 		else if (this->state != WFT_STATE_SUCCESS)
 			this->disable_retry();
 	}
-	else
-		this->get_resp()->end_parsing();
 
 	return true;
 }
