@@ -32,7 +32,8 @@ using namespace protocol;
 
 /**********Client**********/
 
-class __ComplexKafkaTask : public WFComplexClientTask<KafkaRequest, KafkaResponse, std::function<void (__WFKafkaTask *)>>
+class __ComplexKafkaTask : public WFComplexClientTask<KafkaRequest, KafkaResponse,
+													  struct __ComplexKafkaTaskCtx>
 {
 public:
 	__ComplexKafkaTask(int retry_max, __kafka_callback_t&& callback) :
@@ -40,6 +41,7 @@ public:
 	{
 		is_user_request_ = true;
 		is_redirect_ = false;
+		ctx_.kafka_error = 0;
 	}
 
 protected:
@@ -99,6 +101,7 @@ private:
 		}
 	};
 
+	virtual int keep_alive_timeout();
 	virtual int first_timeout();
 	bool has_next();
 	bool check_redirect();
@@ -342,6 +345,14 @@ bool __ComplexKafkaTask::init_success()
 	return true;
 }
 
+int __ComplexKafkaTask::keep_alive_timeout()
+{
+	if (this->get_resp()->get_broker()->get_error())
+		return 0;
+
+	return this->WFComplexClientTask::keep_alive_timeout();
+}
+
 int __ComplexKafkaTask::first_timeout()
 {
 	KafkaRequest *client_req = this->get_req();
@@ -415,9 +426,10 @@ bool __ComplexKafkaTask::check_redirect()
 
 bool __ComplexKafkaTask::process_find_coordinator()
 {
-	if (this->get_resp()->get_cgroup()->get_error())
+	ctx_.kafka_error = this->get_resp()->get_cgroup()->get_error();
+	if (ctx_.kafka_error)
 	{
-		this->error = this->get_resp()->get_cgroup()->get_error();
+		this->error = WFT_ERR_KAFKA_CGROUP_FAILED;
 		this->state = WFT_STATE_TASK_ERROR;
 		return false;
 	}
@@ -453,12 +465,13 @@ bool __ComplexKafkaTask::process_join_group()
 		this->get_req()->set_api_type(Kafka_JoinGroup);
 		break;
 
-	case KAFKA_NONE:
+	case 0:
 		this->get_req()->set_api_type(Kafka_Metadata);
 		break;
 
 	default:
-		this->error = msg->get_cgroup()->get_error();
+		ctx_.kafka_error = msg->get_cgroup()->get_error();
+		this->error = WFT_ERR_KAFKA_CGROUP_FAILED;
 		this->state = WFT_STATE_TASK_ERROR;
 		return false;
 	}
@@ -468,9 +481,10 @@ bool __ComplexKafkaTask::process_join_group()
 
 bool __ComplexKafkaTask::process_sync_group()
 {
-	if (this->get_resp()->get_cgroup()->get_error())
+	ctx_.kafka_error = this->get_resp()->get_cgroup()->get_error();
+	if (ctx_.kafka_error)
 	{
-		this->error = this->get_resp()->get_cgroup()->get_error();
+		this->error = WFT_ERR_KAFKA_CGROUP_FAILED;
 		this->state = WFT_STATE_TASK_ERROR;
 		return false;
 	}
@@ -493,10 +507,11 @@ bool __ComplexKafkaTask::process_metadata()
 		case KAFKA_LEADER_NOT_AVAILABLE:
 			this->get_req()->set_api_type(Kafka_Metadata);
 			return true;
-		case KAFKA_NONE:
+		case 0:
 			break;
 		default:
-			this->error = meta->get_error();
+			ctx_.kafka_error = meta->get_error();
+			this->error = WFT_ERR_KAFKA_META_FAILED;
 			this->state = WFT_STATE_TASK_ERROR;
 			return false;
 		}
@@ -550,11 +565,12 @@ bool __ComplexKafkaTask::process_fetch()
 		case KAFKA_FENCED_LEADER_EPOCH:
 			this->get_req()->set_api_type(Kafka_Metadata);
 			return true;
-		case KAFKA_NONE:
+		case 0:
 		case KAFKA_OFFSET_OUT_OF_RANGE:
 			break;
 		default:
-			this->error = toppar->get_error();
+			ctx_.kafka_error = toppar->get_error();
+			this->error = WFT_ERR_KAFKA_FETCH_FAILED;
 			this->state = WFT_STATE_TASK_ERROR;
 			return false;
 		}
@@ -600,7 +616,7 @@ bool __ComplexKafkaTask::process_produce()
 		case KAFKA_FENCED_LEADER_EPOCH:
 			this->get_req()->set_api_type(Kafka_Metadata);
 			return true;
-		case KAFKA_NONE:
+		case 0:
 			break;
 		default:
 			this->error = toppar->get_error();
@@ -613,9 +629,10 @@ bool __ComplexKafkaTask::process_produce()
 
 bool __ComplexKafkaTask::process_sasl_handshake()
 {
-	if (this->get_resp()->get_broker()->get_error())
+	ctx_.kafka_error = this->get_resp()->get_broker()->get_error();
+	if (ctx_.kafka_error)
 	{
-		this->error = this->get_resp()->get_broker()->get_error();
+		this->error = WFT_ERR_KAFKA_SASL_DISALLOWED;
 		this->state = WFT_STATE_TASK_ERROR;
 		return false;
 	}
@@ -624,9 +641,10 @@ bool __ComplexKafkaTask::process_sasl_handshake()
 
 bool __ComplexKafkaTask::process_sasl_authenticate()
 {
-	if (this->get_resp()->get_broker()->get_error())
+	ctx_.kafka_error = this->get_resp()->get_broker()->get_error();
+	if (ctx_.kafka_error)
 	{
-		this->error = this->get_resp()->get_broker()->get_error();
+		this->error = WFT_ERR_KAFKA_SASL_DISALLOWED;
 		this->state = WFT_STATE_TASK_ERROR;
 	}
 	return false;
@@ -671,14 +689,15 @@ bool __ComplexKafkaTask::has_next()
 	case Kafka_LeaveGroup:
 	case Kafka_DescribeGroups:
 	case Kafka_Heartbeat:
-		this->error = this->get_resp()->get_cgroup()->get_error();
-		if (this->error)
+		ctx_.kafka_error = this->get_resp()->get_cgroup()->get_error();
+		if (ctx_.kafka_error)
+		{
+			this->error = WFT_ERR_KAFKA_CGROUP_FAILED;
 			this->state = WFT_STATE_TASK_ERROR;
+		}
+
 		break;
 	case Kafka_ApiVersions:
-		this->error = errno;
-		if (this->error)
-			this->state = WFT_STATE_TASK_ERROR;
 		break;
 	default:
 		this->state = WFT_STATE_TASK_ERROR;
@@ -730,8 +749,8 @@ bool __ComplexKafkaTask::finish_once()
 		this->get_resp()->duplicate(*this->get_req());
 	}
 
-	if (*get_mutable_ctx())
-		(*get_mutable_ctx())(this);
+	if (ctx_.cb)
+		ctx_.cb(this);
 
 	return true;
 }
