@@ -93,6 +93,11 @@ public:
 									  this->parser) == 0;
 	}
 
+	bool is_header_complete() const
+	{
+		return http_parser_header_complete(this->parser);
+	}
+
 	bool has_connection_header() const
 	{
 		return http_parser_has_connection(this->parser);
@@ -126,7 +131,19 @@ public:
 	 * msg->get_parsed_body(&body, &size);
 	 * msg->append_output_body_nocopy(body, size); */
 	bool append_output_body(const void *buf, size_t size);
+
+	bool append_output_body(const char *buf)
+	{
+		return this->append_output_body(buf, strlen(buf));
+	}
+
 	bool append_output_body_nocopy(const void *buf, size_t size);
+
+	bool append_output_body_nocopy(const char *buf)
+	{
+		return this->append_output_body_nocopy(buf, strlen(buf));
+	}
+
 	void clear_output_body();
 
 	size_t get_output_body_size() const
@@ -173,6 +190,11 @@ public:
 		return this->append_output_body(buf.c_str(), buf.size());
 	}
 
+	bool append_output_body_nocopy(const std::string& buf)
+	{
+		return this->append_output_body_nocopy(buf.c_str(), buf.size());
+	}
+
 protected:
 	http_parser_t *parser;
 	size_t cur_size;
