@@ -47,11 +47,14 @@ public:
 	virtual bool add_produce_record(const std::string& topic, int partition,
 									protocol::KafkaRecord record) = 0;
 
+	virtual bool add_offset_toppar(const protocol::KafkaToppar& toppar) = 0;
+
 	void add_commit_record(const protocol::KafkaRecord& record)
 	{
 		protocol::KafkaToppar toppar;
 		toppar.set_topic_partition(record.get_topic(), record.get_partition());
 		toppar.set_offset(record.get_offset());
+		toppar.set_error(0);
 		this->toppar_list.add_item(std::move(toppar));
 	}
 
@@ -60,6 +63,7 @@ public:
 		protocol::KafkaToppar toppar_t;
 		toppar_t.set_topic_partition(toppar.get_topic(), toppar.get_partition());
 		toppar_t.set_offset(toppar.get_offset());
+		toppar_t.set_error(0);
 		this->toppar_list.add_item(std::move(toppar_t));
 	}
 
@@ -88,6 +92,11 @@ public:
 		return &this->result;
 	}
 
+	int get_kafka_error() const
+	{
+		return this->kafka_error;
+	}
+
 	void set_callback(kafka_callback_t cb)
 	{
 		this->callback = std::move(cb);
@@ -113,6 +122,7 @@ protected:
 	kafka_callback_t callback;
 	kafka_partitioner_t partitioner;
 	int api_type;
+	int kafka_error;
 	int retry_max;
 	bool finish;
 
@@ -123,8 +133,6 @@ private:
 class WFKafkaClient
 {
 public:
-	WFKafkaClient();
-
 	// example: kafka://10.160.23.23:9000
 	// example: kafka://kafka.sogou
 	// example: kafka.sogou:9090
@@ -133,7 +141,7 @@ public:
 
 	int init(const std::string& broker_url, const std::string& group);
 
-	void deinit();
+	int deinit();
 
 	// example: topic=xxx&topic=yyy&api=fetch
 	// example: api=commit
@@ -143,21 +151,21 @@ public:
 
 	WFKafkaTask *create_kafka_task(int retry_max, kafka_callback_t cb);
 
+	void set_config(protocol::KafkaConfig conf);
+
 public:
 	/* If you don't leavegroup manually, rebalance would be triggered */
 	WFKafkaTask *create_leavegroup_task(int retry_max,
 										kafka_callback_t callback);
 
 public:
-	virtual ~WFKafkaClient();
-
 	protocol::KafkaMetaList *get_meta_list();
 
 	protocol::KafkaBrokerList *get_broker_list();
 
 private:
 	class KafkaMember *member;
-	friend class ComplexKafkaTask;
+	friend class KafkaClientTask;
 };
 
 #endif
