@@ -1801,9 +1801,6 @@ int KafkaMessage::append(const void *buf, size_t *size)
 		ret = -1;
 	}
 
-	if (ret == 1 || ret < 0)
-		return ret;
-
 	return ret;
 }
 
@@ -2821,7 +2818,7 @@ int KafkaResponse::parse_response()
 		return -1;
 	}
 
-	return 0;
+	return ret;
 }
 
 static int kafka_meta_parse_broker(void **buf, size_t *size,
@@ -3179,7 +3176,7 @@ int KafkaResponse::parse_produce(void **buf, size_t *size)
 				record = list_entry(pos, KafkaRecord, list);
 				record->set_status(ptr->error);
 
-				if (ptr->error != KAFKA_NONE)
+				if (ptr->error)
 					continue;
 
 				record->set_offset(base_offset++);
@@ -3729,14 +3726,14 @@ int KafkaResponse::parse_apiversions(void **buf, size_t *size)
 int KafkaResponse::parse_saslhandshake(void **buf, size_t *size)
 {
 	std::string mechanism;
+	int16_t error = 0;
 	int32_t cnt, i;
-	int16_t error;
 
 	CHECK_RET(parse_i16(buf, size, &error));
 	if (error != 0)
 	{
-		errno = EBADMSG;
-		return -1;
+		this->broker.get_raw_ptr()->error = error;
+		return 1;
 	}
 
 	CHECK_RET(parse_i32(buf, size, &cnt));
@@ -3749,17 +3746,23 @@ int KafkaResponse::parse_saslhandshake(void **buf, size_t *size)
 			break;
 	}
 
-	if (i >= cnt)
-	{
-		errno = EBADMSG;
-		return -1;
-	}
-
-	if (!this->config.new_client(this->sasl))
+	if (i == cnt)
 	{
 		this->broker.get_raw_ptr()->error = KAFKA_SASL_AUTHENTICATION_FAILED;
-		errno = EBADMSG;
-		return -1;
+		return 1;
+	}
+
+	for (i++; i < cnt; i++)
+		CHECK_RET(parse_string(buf, size, mechanism));
+
+	errno = 0;
+	if (!this->config.new_client(this->sasl))
+	{
+		if (errno)
+			return -1;
+
+		this->broker.get_raw_ptr()->error = KAFKA_SASL_AUTHENTICATION_FAILED;
+		return 1;
 	}
 
 	return 0;
@@ -3768,27 +3771,30 @@ int KafkaResponse::parse_saslhandshake(void **buf, size_t *size)
 int KafkaResponse::parse_saslauthenticate(void **buf, size_t *size)
 {
 	std::string error_message;
-	int16_t error;
+	std::string auth_bytes;
+	int16_t error = 0;
 
 	CHECK_RET(parse_i16(buf, size, &error));
 	CHECK_RET(parse_string(buf, size, error_message));
+	CHECK_RET(parse_bytes(buf, size, auth_bytes));
 
 	if (error != 0)
 	{
-		errno = EBADMSG;
-		return -1;
+		this->broker.get_raw_ptr()->error = error;
+		return 1;
 	}
 
-	std::string auth_bytes;
-	CHECK_RET(parse_bytes(buf, size, auth_bytes));
+	errno = 0;
 	if (this->config.get_raw_ptr()->recv(auth_bytes.c_str(),
 										 auth_bytes.size(),
 										 this->config.get_raw_ptr(),
 										 this->sasl) != 0)
 	{
+		if (errno)
+			return -1;
+
 		this->broker.get_raw_ptr()->error = KAFKA_SASL_AUTHENTICATION_FAILED;
-		errno = EBADMSG;
-		return -1;
+		return 1;
 	}
 
 	return 0;
@@ -3811,7 +3817,7 @@ int KafkaResponse::handle_sasl_continue()
 		ret = this->feedback(iovecs[i].iov_base, iovecs[i].iov_len);
 		if (ret != (int)iovecs[i].iov_len)
 		{
-			if (ret > 0)
+			if (ret >= 0)
 				errno = EAGAIN;
 			return -1;
 		}
@@ -3824,34 +3830,33 @@ int KafkaResponse::append(const void *buf, size_t *size)
 {
 	int ret = KafkaMessage::append(buf, size);
 
-	if (ret == 1)
+	if (ret <= 0)
+		return ret;
+
+	ret = this->parse_response();
+	if (ret != 0)
+		return ret;
+
+	if (this->api_type == Kafka_SaslHandshake)
 	{
-		if (this->parse_response() == 0)
+		this->api_type = Kafka_SaslAuthenticate;
+		this->clear_buf();
+		return this->handle_sasl_continue();
+	}
+	else if (this->api_type == Kafka_SaslAuthenticate)
+	{
+		if (strncasecmp(this->config.get_sasl_mech(), "SCRAM", 5) == 0)
 		{
-			if (this->api_type == Kafka_SaslHandshake)
-			{
-				this->api_type = Kafka_SaslAuthenticate;
-				this->clear_buf();
-				ret = this->handle_sasl_continue();
-			}
-			else if (this->api_type == Kafka_SaslAuthenticate)
-			{
-				if (strncasecmp(this->config.get_sasl_mech(), "SCRAM", 5) == 0)
-				{
-					this->clear_buf();
-					if (this->sasl->scram.state !=
-							KAFKA_SASL_SCRAM_STATE_CLIENT_FINISHED)
-						ret = this->handle_sasl_continue();
-					else
-						this->sasl->status = 1;
-				}
-			}
+			this->clear_buf();
+			if (this->sasl->scram.state !=
+					KAFKA_SASL_SCRAM_STATE_CLIENT_FINISHED)
+				return this->handle_sasl_continue();
+			else
+				this->sasl->status = 1;
 		}
-		else
-			ret = -1;
 	}
 
-	return ret;
+	return 1;
 }
 
 }
